@@ -1,6 +1,6 @@
 /*
 Weborf
-Copyright (C) 2017  Salvo "LtWorf" Tomaselli
+Copyright (C) 2017-2018  Salvo "LtWorf" Tomaselli
 
 Weborf is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -133,22 +133,20 @@ static inline void set_connection_props(connection_t *connection_prop) {
 
 static inline void handle_requests(char* buf,buffered_read_t * read_b,int * bufFull,connection_t* connection_prop,long int id) {
     int from;
-    int sock=connection_prop->sock;
+    fd_t sock = connection_prop->sock;
     char *lasts;//Used by strtok_r
 
-    short int r;//Readed char
-    char *end;//Pointer to header's end
+    short int r; //Readed char
+    char *end; //Pointer to header's end
 
     while (true) { //Infinite cycle to handle all pipelined requests
         if ((*bufFull)!=0) {
             memset(buf,0,(*bufFull));//Sets to 0 the buffer, only the part used for the previous request in the same connection
-            (*bufFull)=0;//bufFull-(end-buf+4);
+            (*bufFull)=0;
         }
         from=0;
 
         while ((end=strstr(buf+from,"\r\n\r"))==NULL) { //Determines if there is a \r\n\r which is an ending sequence
-            //ssize_t rsize=buffer_strstr(sock,read_b,"\r\n\r\n");
-            //r=buffer_read(sock, buf+(*bufFull),rsize+strlen("\r\n\r\n"),read_b);//Reads 2 char and adds to the buffer
             r=buffer_read(sock, buf+(*bufFull),2,read_b);//Reads 2 char and adds to the buffer
 
             if (r<=0) { //Connection closed or error
@@ -159,23 +157,19 @@ static inline void handle_requests(char* buf,buffered_read_t * read_b,int * bufF
                 from=(*bufFull);
             }
 
-            //TODO remove this crap!!
-            //if ((*bufFull)!=0) { //Removes Cr Lf from beginning
             (*bufFull)+=r;//Sets the end of the user buffer (may contain more than one header)
-            //} else if (buf[*bufFull]!='\n' && buf[*bufFull]!='\r') {
-            //    (*bufFull)+=r;
-            //}
 
             //Buffer full and still no valid http header
             if ((*bufFull)>=INBUFFER) goto bad_request;
 
         }
 
-        if (strstr(buf+from,"\r\n\r\n")==NULL) {//If we didn't read yet the lst \n of the ending sequence, we read it now, so it won't disturb the next request
-            *bufFull+=buffer_read(sock, buf+*bufFull,1,read_b);//Reads 1 char and adds to the buffer
+        //If we didn't read yet the lst \n of the ending sequence, we read it now, so it won't disturb the next request
+        if (strstr(buf+from,"\r\n\r\n")==NULL) {
+            *bufFull+=buffer_read(sock, buf+*bufFull,1,read_b);
         }
 
-        end[2]='\0';//Terminates the header, leaving a final \r\n in it
+        end[2]='\0'; //Terminates the header, leaving a final \r\n in it
 
         //Finds out request's kind
         if (strncmp(buf,"GET",strlen("GET"))==0) connection_prop->method_id=GET;
@@ -204,13 +198,15 @@ static inline void handle_requests(char* buf,buffered_read_t * read_b,int * bufF
         //Stores the parameters of the request
         set_connection_props(connection_prop);
 
-        if (send_page(read_b, connection_prop)<0) {
+        if (send_page(read_b, connection_prop) < 0) {
 #ifdef REQUESTDBG
-            syslog(LOG_INFO,"%s - FAILED - %s %s",connection_prop->ip_addr,connection_prop->method,connection_prop->page);
+            syslog(LOG_INFO,
+                   "%s - FAILED - %s %s",
+                   connection_prop->ip_addr,
+                   connection_prop->method,
+                   connection_prop->page);
 #endif
-
-            close(sock);
-            return;//Unable to send an error
+            return; //Unable to send an error
         }
 
 #ifdef REQUESTDBG
@@ -234,7 +230,6 @@ bad_request:
 #ifdef REQUESTDBG
     syslog(LOG_INFO, "%s - %d", connection_prop->ip_addr, connection_prop->status_code);
 #endif
-    close(sock);
     return;
 }
 
@@ -286,40 +281,64 @@ void * instance(void * nulla) {
     int addr_l=sizeof(struct sockaddr_in);
 #endif
 
-    if (mime_init(&thread_prop.mime_token)!=0 || buffer_init(&read_b,BUFFERED_READER_SIZE)!=0 || buf==NULL || connection_prop.strfile==NULL) { //Unable to allocate the buffer
+    if (mime_init(&thread_prop.mime_token) != 0 || buffer_init(&read_b, BUFFERED_READER_SIZE) != 0 || buf == NULL || connection_prop.strfile == NULL) { //Unable to allocate the buffer
 #ifdef SERVERDBG
-        syslog(LOG_CRIT,"Not enough memory to allocate buffers for new thread");
+        syslog(LOG_CRIT, "Not enough memory to allocate buffers for new thread");
 #endif
         goto release_resources;
     }
 
     //Start accepting sockets
-    change_free_thread(thread_prop.id,1,0);
+    change_free_thread(thread_prop.id, 1, 0);
 
     while (true) {
         q_get(&queue, &sock);//Gets a socket from the queue
-        change_free_thread(thread_prop.id,-1,0);//Sets this thread as busy
+        change_free_thread(thread_prop.id, -1, 0);//Sets this thread as busy
 
         if (sock<0) { //Was not a socket but a termination order
             goto release_resources;
         }
 
-        connection_prop.sock=sock;//Assigned socket into the struct
-        net_getpeername(sock,connection_prop.ip_addr);
+        net_getpeername(sock, connection_prop.ip_addr);
 
-#ifdef THREADDBG
-        syslog(LOG_DEBUG,"Thread %ld: Reading from socket",thread_prop.id);
-#endif
-        handle_requests(buf,&read_b,&bufFull,&connection_prop,thread_prop.id);
+#ifdef HAVE_LIBSSL
+        connection_prop.sock.fd = sock;
 
-#ifdef THREADDBG
-        syslog(LOG_DEBUG,"Thread %ld: Closing socket with client",thread_prop.id);
+        if (weborf_conf.sslctx) {
+            connection_prop.sock.ssl = SSL_new(weborf_conf.sslctx);
+            SSL_set_fd(connection_prop.sock.ssl, sock);
+            if (SSL_accept(connection_prop.sock.ssl) != 1) {
+                syslog(LOG_INFO, "SSL connection failed from %s", connection_prop.ip_addr);
+                goto closeconnection;
+            }
+        } else {
+            connection_prop.sock.ssl = NULL;
+        }
+#else
+    connection_prop.sock = sock;
 #endif
 
-        close(sock);//Closing the socket
-        buffer_reset (&read_b);
 
-        change_free_thread(thread_prop.id,1,0);//Sets this thread as free
+#ifdef THREADDBG
+        syslog(LOG_DEBUG, "Thread %ld: Reading from socket", thread_prop.id);
+#endif
+        handle_requests(buf, &read_b, &bufFull, &connection_prop, thread_prop.id);
+
+closeconnection:
+#ifdef THREADDBG
+        syslog(LOG_DEBUG, "Thread %ld: Closing socket with client", thread_prop.id);
+#endif
+
+#ifdef HAVE_LIBSSL
+    if (weborf_conf.sslctx) {
+        SSL_shutdown(connection_prop.sock.ssl);
+        SSL_free(connection_prop.sock.ssl);
+    }
+#endif
+        close(sock); //Closing the socket
+        buffer_reset(&read_b);
+
+        change_free_thread(thread_prop.id, 1, 0);//Sets this thread as free
     }
 
 
@@ -348,7 +367,7 @@ Auth provider has to check for the file's size and refuse it if it is the case.
 This function will not work if there is no auth provider.
 */
 int read_file(connection_t* connection_prop,buffered_read_t* read_b) {
-    int sock=connection_prop->sock;
+    fd_t sock = connection_prop->sock;
     if (weborf_conf.authsock==NULL) {
         return ERR_FORBIDDEN;
     }
@@ -482,7 +501,7 @@ This function determines the requested page and sends it
 http_param is a string containing parameters of the HTTP request
 */
 static int send_page(buffered_read_t* read_b, connection_t* connection_prop) {
-    int retval=0;//Return value after sending the page
+    int retval = 0;//Return value after sending the page
     string_t post_param; //Contains POST data
     post_param.data = NULL;
     post_param.len = 0;
@@ -497,9 +516,14 @@ static int send_page(buffered_read_t* read_b, connection_t* connection_prop) {
         goto escape;
     }
 
-    connection_prop->strfile_len = snprintf(connection_prop->strfile,URI_LEN,"%s%s",connection_prop->basedir,connection_prop->page);//Prepares the string
+    connection_prop->strfile_len = snprintf(
+        connection_prop->strfile,
+        URI_LEN,
+        "%s%s",
+        connection_prop->basedir,
+        connection_prop->page);
 
-    if (connection_prop->method_id>=PUT) {//Methods from PUT to other uncommon ones :-D
+    if (connection_prop->method_id >= PUT) {//Methods from PUT to other uncommon ones :-D
         switch (connection_prop->method_id) {
         case PUT:
             retval=read_file(connection_prop, read_b);
@@ -528,7 +552,7 @@ static int send_page(buffered_read_t* read_b, connection_t* connection_prop) {
         goto escape;
     }
 
-    if (connection_prop->method_id==POST)
+    if (connection_prop->method_id == POST)
         post_param = read_post_data(connection_prop,read_b);
 
     if ((connection_prop->strfile_fd=open(connection_prop->strfile,O_RDONLY | O_LARGEFILE))<0) {
@@ -657,7 +681,7 @@ This function writes on the specified socket an html page containing the list of
 specified directory.
 */
 int write_dir(char* real_basedir,connection_t* connection_prop) {
-    int sock=connection_prop->sock;
+    fd_t sock = connection_prop->sock;
 
     /*
     WARNING
@@ -731,7 +755,7 @@ int write_dir(char* real_basedir,connection_t* connection_prop) {
             connection_prop->strfile_stat.st_mtime,
             connection_prop
         );
-        write(sock, html, pagelen);
+        myio_write(sock, html, pagelen);
 
         //Write item in cache
         cache_store_item(0, connection_prop, html, pagelen);
@@ -752,7 +776,7 @@ sock is the socket
 */
 #ifdef __COMPRESSION
 static inline int write_compressed_file(connection_t* connection_prop ) {
-    int sock=connection_prop->sock;
+    fd_t sock = connection_prop->sock;
 
     if (
         connection_prop->strfile_stat.st_size>SIZE_COMPRESS_MIN &&
@@ -913,7 +937,7 @@ The file sent is the one specified by strfile_fd, and it will not be closed afte
 */
 int write_file(connection_t* connection_prop) {
 
-    int sock=connection_prop->sock;
+    fd_t sock = connection_prop->sock;
 
     char a[RBUFFER+MIMETYPELEN+16]; //Buffer for Range, Content-Range headers, and reading if-none-match from header
 
@@ -938,14 +962,14 @@ int write_file(connection_t* connection_prop) {
     }*/
 
     //Copy file using descriptors; from to and size
-    return fd_copy(connection_prop->strfile_fd,sock,count);
+    return fd_copy(fd2fd_t(connection_prop->strfile_fd), sock, count);
 }
 
 /**
 Sends a request for authentication
 */
 int request_auth(connection_t *connection_prop) {
-    int sock=connection_prop->sock;
+    fd_t sock = connection_prop->sock;
     char * descr = connection_prop->page;
     connection_prop->status_code=401;
 
@@ -967,13 +991,13 @@ int request_auth(connection_t *connection_prop) {
     int head_len = snprintf(head,HEADBUF,"HTTP/1.1 401 Authorization Required\r\nServer: " SIGNATURE "\r\nContent-Length: %d\r\nWWW-Authenticate: Basic realm=\"%s\"\r\n\r\n",page_len,descr);
 
     //Sends the header
-    if (write (sock,head,head_len)!=head_len) {
+    if (myio_write (sock,head,head_len)!=head_len) {
         free(head);
         return ERR_SOCKWRITE;
     }
 
     //Sends the page
-    if (write(sock,page,page_len)!=page_len) {
+    if (myio_write(sock,page,page_len)!=page_len) {
         free(head);
         return ERR_SOCKWRITE;
     }
@@ -987,8 +1011,7 @@ int request_auth(connection_t *connection_prop) {
 Sends an error to the client
 */
 int send_err(connection_t *connection_prop, int err, char* descr) {
-    int sock=connection_prop->sock;
-
+    fd_t sock = connection_prop->sock;
     connection_prop->status_code = err; //Sets status code, for the logs
 
     //Buffer for both header and page
@@ -1010,13 +1033,13 @@ int send_err(connection_t *connection_prop, int err, char* descr) {
     int head_len = snprintf(head,HEADBUF,"HTTP/1.1 %d %s\r\nServer: " SIGNATURE "\r\nContent-Length: %d\r\nContent-Type: text/html\r\n\r\n",err,descr ,(int)page_len);
 
     //Sends the http header
-    if (write (sock,head,head_len)!=head_len) {
+    if (myio_write(sock, head, head_len) != head_len) {
         free(head);
         return ERR_SOCKWRITE;
     }
 
     //Sends the html page
-    if (write(sock,page,page_len)!=page_len) {
+    if (myio_write(sock, page, page_len) != page_len) {
         free(head);
         return ERR_SOCKWRITE;
     }
@@ -1031,7 +1054,7 @@ or NULL if there was no data.
 If it doesn't return a null value, the returned pointer must be freed.
 */
 string_t read_post_data(connection_t* connection_prop, buffered_read_t* read_b) {
-    int sock=connection_prop->sock;
+    fd_t sock = connection_prop->sock;
     string_t res;
     res.len=0;
     res.data=NULL;
@@ -1039,7 +1062,7 @@ string_t read_post_data(connection_t* connection_prop, buffered_read_t* read_b)
     //Buffer for field's value
     char a[NBUFFER];
     //Gets the value
-    char *content_length="Content-Length";
+    char *content_length = "Content-Length";
 
     //If there is a request body
     if (get_param_value(connection_prop->http_param, content_length, a, NBUFFER, strlen(content_length))) {
@@ -1116,7 +1139,7 @@ needed, according to keep_alive and protocol_version of connection_prop
 
 */
 int send_http_header(int code, unsigned long long int size,char* headers,bool content,time_t timestamp,connection_t* connection_prop) {
-    int sock=connection_prop->sock;
+    fd_t sock = connection_prop->sock;
     int len_head,wrote;
     char *head=malloc(HEADBUF);
     char* h_ptr=head;
@@ -1191,7 +1214,7 @@ int send_http_header(int code, unsigned long long int size,char* headers,bool co
     //head+=len_head; Not necessary because the snprintf was the last one
     left_head-=len_head;
 
-    wrote=write (sock,h_ptr,HEADBUF-left_head);
+    wrote = myio_write(sock, h_ptr, HEADBUF - left_head);
     free(h_ptr);
     if (wrote!=HEADBUF-left_head) return ERR_BRKPIPE;
     return 0;
@@ -1233,7 +1256,7 @@ static int tar_send_dir(connection_t* connection_prop) {
 
     if (pid==0) { //child, executing tar
         fclose (stdout); //Closing the stdout
-        if (dup(connection_prop->sock) == -1) { //Redirects the stdout
+        if (dup(myio_getfd(connection_prop->sock)) == -1) { //Redirects the stdout
             exit(1);
         }
         errno = 0;
@@ -1263,12 +1286,10 @@ will use 0 as socket and exit after.
 It is almost a copy of instance()
 */
 void inetd() {
-
     thread_prop_t thread_prop;  //Server's props
     int bufFull=0;                                  //Amount of buf used
     connection_t connection_prop;                   //Struct to contain properties of the connection
     buffered_read_t read_b;                         //Buffer for buffered reader
-    int sock=connection_prop.sock=0;                //Socket with the client,using normal file descriptor 0
     char * buf=calloc(INBUFFER+1,sizeof(char));     //Buffer to contain the HTTP request
     connection_prop.strfile=malloc(URI_LEN);        //buffer for filename
 
@@ -1289,26 +1310,32 @@ void inetd() {
     int addr_l=sizeof(struct sockaddr_in);
 #endif
 
-    if (mime_init(&thread_prop.mime_token)!=0 || buffer_init(&read_b,BUFFERED_READER_SIZE)!=0 || buf==NULL || connection_prop.strfile==NULL) { //Unable to allocate the buffer
+    if (mime_init(&thread_prop.mime_token)!=0 || buffer_init(&read_b, BUFFERED_READER_SIZE)!=0 || buf==NULL || connection_prop.strfile==NULL) {
 #ifdef SERVERDBG
         syslog(LOG_CRIT,"Not enough memory to allocate buffers for new thread");
 #endif
-        goto release_resources;
+        exit(1);
     }
 
-    net_getpeername(sock,connection_prop.ip_addr);
+    net_getpeername(0, connection_prop.ip_addr);
+
+#ifdef HAVE_LIBSSL
+        connection_prop.sock.fd = 0;
+
+        if (weborf_conf.sslctx) {
+            connection_prop.sock.ssl = SSL_new(weborf_conf.sslctx);
+            SSL_set_fd(connection_prop.sock.ssl, 0);
+            if (SSL_accept(connection_prop.sock.ssl) != 1) {
+                syslog(LOG_INFO, "SSL connection failed from %s", connection_prop.ip_addr);
+                exit(1);
+            }
+        } else {
+            connection_prop.sock.ssl = NULL;
+        }
+#else
+    connection_prop.sock = 0;
+#endif
 
     handle_requests(buf,&read_b,&bufFull,&connection_prop,thread_prop.id);
-    //close(sock);//Closing the socket
-    //buffer_reset (&read_b);
-
-release_resources:
     exit(0);
-    /* No need to free memory and resources
-    free(buf);
-    free(connection_prop.strfile);
-    buffer_free(&read_b);
-    mime_release(thread_prop.mime_token);
-    */
-    return;
 }
