@@ -29,7 +29,6 @@ import qweborf.nhelper as nhelper
 
 class qweborfForm (QtWidgets.QWidget):
 
-
     DBG_DEFAULT = 0
     DBG_WARNING = 1
     DBG_ERROR = 2
@@ -49,6 +48,12 @@ class qweborfForm (QtWidgets.QWidget):
             self.ui.chkDav.setEnabled(False)
             self.ui.chkWrite.setEnabled(False)
 
+        if not self.weborf.https:
+            self.ui.txtKey.setEnabled(False)
+            self.ui.txtCert.setEnabled(False)
+            self.ui.cmdOpenCert.setEnabled(False)
+            self.ui.cmdOpenKey.setEnabled(False)
+
         # Listing addresses
         for i in nhelper.getaddrs(self.weborf.ipv6):
             self.ui.cmbAddress.addItem(i, None)
@@ -91,7 +96,7 @@ class qweborfForm (QtWidgets.QWidget):
 
     def about(self):
 
-        self.logger('<hr><strong>Qweborf 0.14</strong>')
+        self.logger('<hr><strong>Qweborf 0.15</strong>')
         self.logger('This program comes with ABSOLUTELY NO WARRANTY.'
                     ' This is free software, and you are welcome to redistribute it under certain conditions.'
                     ' For details see the <a href="http://www.gnu.org/licenses/gpl.html">GPLv3 Licese</a>.')
@@ -118,6 +123,13 @@ class qweborfForm (QtWidgets.QWidget):
             options['username'] = None
             options['password'] = None
 
+        if self.ui.txtKey.text() or self.ui.txtCert.text():
+            options['key'] = self.ui.txtKey.text()
+            options['cert'] = self.ui.txtCert.text()
+        else:
+            options['key'] = None
+            options['cert'] = None
+
         options['port'] = self.ui.spinPort.value()
 
         options['dav'] = self.ui.chkDav.isChecked()
@@ -155,6 +167,26 @@ class qweborfForm (QtWidgets.QWidget):
                     logentry='Public address: <a href="%s">%s</a>' % (url,url)
                     self.logger(logentry)
 
+    def select_cert(self):
+        fname = QtWidgets.QFileDialog.getOpenFileName(
+            self,
+            'Certificate',
+            self.ui.txtCert.text(),
+            filter='PEM certificates(*.pem);;All files(*)',
+        )[0]
+        if fname:
+            self.ui.txtCert.setText(fname)
+
+
+    def select_key(self):
+        fname = QtWidgets.QFileDialog.getOpenFileName(
+            self,
+            'Certificate',
+            self.ui.txtKey.text(),
+        )[0]
+        if fname:
+            self.ui.txtKey.setText(fname)
+
     def select_path(self):
         current = self.ui.txtPath.text()
         if len(current) == 0:
