@@ -1,6 +1,6 @@
 /*
 Weborf
-Copyright (C) 2007  Salvo "LtWorf" Tomaselli
+Copyright (C) 2007-2018  Salvo "LtWorf" Tomaselli
 
 Weborf is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -274,7 +274,7 @@ Prints version information
 */
 void version() {
     printf("Weborf %s\n"
-           "Copyright (C) 2007 Salvo 'LtWorf' Tomaselli.\n"
+           "Copyright (C) 2007-2018 Salvo 'LtWorf' Tomaselli.\n"
            "This is free software.  You may redistribute copies of it under the terms of\n"
            "the GNU General Public License <http://www.gnu.org/licenses/gpl.html>.\n"
            "There is NO WARRANTY, to the extent permitted by law.\n\n"
@@ -324,6 +324,15 @@ void print_capabilities() {
     "\n"
     );
 
+    printf("https:"
+#ifdef HAVE_LIBSSL
+    "yes"
+#else
+    "no"
+#endif
+    "\n"
+    );
+
     exit(0);
 }
 
@@ -370,9 +379,10 @@ void help() {
            "  -g  --gid     followed by a valid gid\n"
            "  -V, --virtual list of virtualhosts in the form host=basedir, comma-separated\n"
            "  -v, --version print program version\n"
-           "  -x, --noexec  tells weborf to send each file instead of executing scripts\n\n"
-
-
+           "  -x, --noexec  tells weborf to send each file instead of executing scripts\n"
+           "  -S, --cert    the certificate to use\n"
+           "  -K, --key     the private key to use with the certificate\n"
+           "\n"
            "Report bugs here https://bugs.launchpad.net/weborf\n"
            "or to " PACKAGE_BUGREPORT "\n");
     exit(0);
